<?php
// src/OgBootBundle/Controller/OgBootController.php

namespace App\OgBootBundle\Controller;
use OpenApi\Annotations as OA;
use Psr\Log\LoggerInterface;
use App\OgBootBundle\Service\CurlRequestService;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\Response;
use Symfony\Component\Routing\Annotation\Route;
use Symfony\Component\HttpFoundation\JsonResponse;
use Exception;


class OgBootController
{
    private $logger;
    /**
     * @OA\Info(title="Ogboot API", version="1.0")
     */

    private $curlRequestService;

    public function __construct(CurlRequestService $curlRequestService, LoggerInterface $logger)
    {
        $this->curlRequestService = $curlRequestService;
        $this->logger = $logger;
    }

/*Tabla de Contenido
Obtener configuración de ogboot - GET /ogboot/config x
Obtener datos de rendimiento - GET /ogboot/status x
Mostrar información de todos los clientes ogLive instalados - GET /ogboot/oglives x 
Mostrar información de un cliente ogLive instalado - GET /ogboot/oglives/{Index|Dir} x
Mostrar información del cliente ogLive predeterminado - GET /ogboot/oglives/default x
Cambiar ogLive predeterminado - POST /ogboot/oglives/default/{Index} x 
Instalar nuevo cliente ogLive desde imagen descargada - POST /ogboot/oglive/{Index/iso} ----
Desinstalar cliente ogLive y eliminar imagen - DELETE /ogboot/oglives/{Index/iso} -------
Regenerar archivo de información de los ogLive - PUT /ogboot/oglives
Mostrar menú de descarga de imagen de ogLive - GET /ogboot/images/download X 

/////////////////////////////////////////////

Obtener todos los archivos de arranque - GET /ogboot/pxes
Obtener archivo de arranque - GET /ogboot/clients/pxes/{mac}
Crear archivo de arranque - POST /ogboot/pxes
Eliminar archivo de arranque - DELETE /ogboot/clients/pxes
Obtener todas las plantillas - GET /ogboot/pxe-templates
Obtener contenido de la plantilla - GET /ogboot/pxe-templates/{nombre}
Crear plantilla - POST /ogboot/pxe-templates
Regenerar plantilla - PUT /ogboot/pxe-templates
*/

    /**
    * @Route("/ogboot/help", name="help", methods={"GET"})
    */
    public function help(): Response
    {

        # $result = $this->curlRequestService->common_request(OG_REST_CMD_POWEROFF, POST, [OG_REST_PARAM_CLIENTS => $ips]);
        $result = $this->curlRequestService->callOgLive("help");

        if ($result) {
            return new Response($result, Response::HTTP_OK);
        } else {
            return new Response('Failed', Response::HTTP_INTERNAL_SERVER_ERROR);
        }
    }

/**
 * @Route("/ogboot/config", name="config", methods={"GET"})
 * @OA\Get(
 *     path="/ogboot/config",
 *     summary="Get ogboot configuration",
 *     @OA\Response(
 *         response=200,
 *         description="Successful operation",
 *         @OA\JsonContent(
 *             type="object",
 *             @OA\Property(
 *                 property="config-file",
 *                 type="string",
 *                 description="Configuration file path"
 *             ),
 *             @OA\Property(
 *                 property="download-url",
 *                 type="string",
 *                 description="ogLive download URL"
 *             ),
 *             @OA\Property(
 *                 property="download-dir",
 *                 type="string",
 *                 description="ogLive download directory"
 *             ),
 *             @OA\Property(
 *                 property="install-dir",
 *                 type="string",
 *                 description="ogLive installation directory"
 *             ),
 *             @OA\Property(
 *                 property="default-name",
 *                 type="string",
 *                 description="Default ogLive name"
 *             ),
 *             @OA\Property(
 *                 property="min-release",
 *                 type="string",
 *                 description="Minimum compatibility release"
 *             )
 *         )
 *     ),
 *     @OA\Response(
 *         response=500,
 *         description="Failed to retrieve configuration"
 *     )
 * )
 */
    public function config(): Response
    {

        # $result = $this->curlRequestService->common_request(OG_REST_CMD_POWEROFF, POST, [OG_REST_PARAM_CLIENTS => $ips]);
        $result = $this->curlRequestService->callOgLive("config");

        if ($result) {
            return new Response($result, Response::HTTP_OK);
        } else {
            return new Response('Failed', Response::HTTP_INTERNAL_SERVER_ERROR);
        }
    }

/**
 * @Route("/ogboot/status", name="status", methods={"GET"})
 * @OA\Get(
 *     path="/ogboot/status",
 *     summary="Get ogboot status",
 *     @OA\Response(
 *         response=200,
 *         description="Successful operation",
 *         @OA\JsonContent(
 *             type="object",
 *             @OA\Property(
 *                 property="message",
 *                 type="string",
 *                 description="Status message indicating any issues detected"
 *             )
 *         )
 *     ),
 *     @OA\Response(
 *         response=500,
 *         description="Failed to retrieve status"
 *     )
 * )
 */
    public function status(): Response
    {

        # $result = $this->curlRequestService->common_request(OG_REST_CMD_POWEROFF, POST, [OG_REST_PARAM_CLIENTS => $ips]);
        $result = $this->curlRequestService->callOgLive("check");

        if ($result) {
            return new Response($result, Response::HTTP_OK);
        } else {
            return new Response('Failed', Response::HTTP_INTERNAL_SERVER_ERROR);
        }
    }

/**
 * @Route("/ogboot/oglives", name="getOglives", methods={"GET"})
 * @OA\Get(
 *     path="/ogboot/oglives",
 *     summary="Get information of all installed ogLive clients",
 *     @OA\Response(
 *         response=200,
 *         description="Successful operation",
 *         @OA\JsonContent(
 *             type="array",
 *             @OA\Items(
 *                 type="object",
 *                 @OA\Property(
 *                     property="distribution",
 *                     type="string",
 *                     description="Distribution name of the ogLive client"
 *                 ),
 *                 @OA\Property(
 *                     property="kernel",
 *                     type="string",
 *                     description="Kernel version of the ogLive client"
 *                 ),
 *                 @OA\Property(
 *                     property="architecture",
 *                     type="string",
 *                     description="Architecture of the ogLive client"
 *                 ),
 *                 @OA\Property(
 *                     property="revision",
 *                     type="string",
 *                     description="Revision of the ogLive client"
 *                 ),
 *                 @OA\Property(
 *                     property="directory",
 *                     type="string",
 *                     description="Directory name of the ogLive client"
 *                 ),
 *                 @OA\Property(
 *                     property="iso",
 *                     type="string",
 *                     description="ISO file name of the ogLive client"
 *                 )
 *             )
 *         )
 *     ),
 *     @OA\Response(
 *         response=500,
 *         description="Failed to retrieve information of installed ogLive clients"
 *     )
 * )
 */
    public function getOglives(): Response
    {

        # $result = $this->curlRequestService->common_request(OG_REST_CMD_POWEROFF, POST, [OG_REST_PARAM_CLIENTS => $ips]);
        $result = $this->curlRequestService->callOgLive("show all");

        if ($result) {
            return new Response($result, Response::HTTP_OK);
        } else {
            return new Response('Failed', Response::HTTP_INTERNAL_SERVER_ERROR);
        }
    }

/**
 * @Route("/ogboot/oglives/default", name="getOgliveDefault", methods={"GET"})
 * @OA\Get(
 *     path="/ogboot/oglives/default",
 *     summary="Get information of the default ogLive client",
 *     @OA\Response(
 *         response=200,
 *         description="Successful operation",
 *         @OA\JsonContent(
 *             type="object",
 *             @OA\Property(
 *                 property="distribution",
 *                 type="string",
 *                 description="Distribution name of the default ogLive client"
 *             ),
 *             @OA\Property(
 *                 property="kernel",
 *                 type="string",
 *                 description="Kernel version of the default ogLive client"
 *             ),
 *             @OA\Property(
 *                 property="architecture",
 *                 type="string",
 *                 description="Architecture of the default ogLive client"
 *             ),
 *             @OA\Property(
 *                 property="revision",
 *                 type="string",
 *                 description="Revision of the default ogLive client"
 *             ),
 *             @OA\Property(
 *                 property="directory",
 *                 type="string",
 *                 description="Directory name of the default ogLive client"
 *             ),
 *             @OA\Property(
 *                 property="iso",
 *                 type="string",
 *                 description="ISO file name of the default ogLive client"
 *             )
 *         )
 *     ),
 *     @OA\Response(
 *         response=500,
 *         description="Failed to retrieve information of the default ogLive client"
 *     )
 * )
 */

    public function getOgliveDefault(Request $request): Response
    {
        # $result = $this->curlRequestService->common_request(OG_REST_CMD_POWEROFF, POST, [OG_REST_PARAM_CLIENTS => $ips]);
        $result = $this->curlRequestService->callOgLive("show default");

        if ($result) {
            return new Response($result, Response::HTTP_OK);
        } else {
            return new Response('Failed', Response::HTTP_INTERNAL_SERVER_ERROR);
        }
    }

/**
 * @Route("/ogboot/oglives/{id}", name="getOglive", methods={"GET"})
 * @OA\Get(
 *     path="/ogboot/oglives/{id}",
 *     summary="Get information of an installed ogLive client",
 *     @OA\Parameter(
 *         name="id",
 *         in="path",
 *         description="Index or Directory of the installed ogLive client",
 *         required=true,
 *         @OA\Schema(
 *             type="string"
 *         )
 *     ),
 *     @OA\Response(
 *         response=200,
 *         description="Successful operation",
 *         @OA\JsonContent(
 *             type="object",
 *             @OA\Property(
 *                 property="distribution",
 *                 type="string",
 *                 description="Distribution name of the installed ogLive client"
 *             ),
 *             @OA\Property(
 *                 property="kernel",
 *                 type="string",
 *                 description="Kernel version of the installed ogLive client"
 *             ),
 *             @OA\Property(
 *                 property="architecture",
 *                 type="string",
 *                 description="Architecture of the installed ogLive client"
 *             ),
 *             @OA\Property(
 *                 property="revision",
 *                 type="string",
 *                 description="Revision of the installed ogLive client"
 *             ),
 *             @OA\Property(
 *                 property="directory",
 *                 type="string",
 *                 description="Directory name of the installed ogLive client"
 *             ),
 *             @OA\Property(
 *                 property="iso",
 *                 type="string",
 *                 description="ISO file name of the installed ogLive client"
 *             )
 *         )
 *     ),
 *     @OA\Response(
 *         response=404,
 *         description="ogLive client not found"
 *     )
 * )
 */

    public function getOglive(Request $request): Response
    {
        $id = (int) $request->get('id');
        # $result = $this->curlRequestService->common_request(OG_REST_CMD_POWEROFF, POST, [OG_REST_PARAM_CLIENTS => $ips]);
        $result = $this->curlRequestService->callOgLive("show $id");

        if ($result) {
            return new Response($result, Response::HTTP_OK);
        } else {
            return new Response('Failed', Response::HTTP_INTERNAL_SERVER_ERROR);
        }
    }

/**
 * @Route("/ogboot/oglives/default/{id}", name="setOgliveDefault", methods={"POST"})
 * @OA\Post(
 *     path="/ogboot/oglives/default/{id}",
 *     summary="Change default ogLive",
 *     @OA\Parameter(
 *         name="id",
 *         in="path",
 *         description="Index of the ogLive client to set as default",
 *         required=true,
 *         @OA\Schema(
 *             type="string"
 *         )
 *     ),
 *     @OA\Response(
 *         response=200,
 *         description="Default ogLive changed successfully",
 *         @OA\JsonContent(
 *             type="object",
 *             @OA\Property(property="success", type="string")
 *         )
 *     ),
 *     @OA\Response(
 *         response=404,
 *         description="ogLive client not found"
 *     )
 * )
 */

    public function setOgliveDefault(Request $request): Response
    {
        $id = (int) $request->get('id');
        # $result = $this->curlRequestService->common_request(OG_REST_CMD_POWEROFF, POST, [OG_REST_PARAM_CLIENTS => $ips]);
        $result = $this->curlRequestService->callOgLive("set-default $id");

        if ($result) {
            return new Response($result, Response::HTTP_OK);
        } else {
            return new Response('Failed', Response::HTTP_INTERNAL_SERVER_ERROR);
        }
    }


    

/**
 * @Route("/ogboot/images/download", name="getDownloadMenu", methods={"GET"})
 * @OA\Get(
 *     path="/ogboot/images/download",
 *     summary="Get available ISO images for download",
 *     @OA\Response(
 *         response=200,
 *         description="List of available ISO images",
 *         @OA\JsonContent(
 *             type="array",
 *             @OA\Items(
 *                 type="object",
 *                 @OA\Property(property="id", type="string"),
 *                 @OA\Property(property="filename", type="string"),
 *                 @OA\Property(property="installed", type="boolean"),
 *                 @OA\Property(property="compatible", type="boolean")
 *             )
 *         )
 *     )
 * )
 */

    public function getDownloadMenu(): Response
    {

        $downloadsOutput = $this->curlRequestService->callOgLive("download");


        $downloads = explode("\n", trim($downloadsOutput));
        $result = [];
        
        foreach ($downloads as $download) {

            $pattern = '/^(\d+)\) (\(\*\))?\s?(\(\+\))?\s?(.+\.iso)$/';
        

            preg_match($pattern, $download, $matches);

            if (!empty($matches)) {
                $id = $matches[1];
                $installed = !empty($matches[2]);
                $compatible = !empty($matches[3]);
                $filename = $matches[4];

                $result[] = [
                    'id' => $id,
                    'filename' => $filename,
                    'installed' => $installed,
                    'compatible' => $compatible
                ];
            }
        }

        $jsonDownloads = json_encode($result);


        return new Response($jsonDownloads, Response::HTTP_OK, [
            'Content-Type' => 'application/json'
        ]);
    }

    /**
     * @Route("/ogboot/oglives/{isoName}", name="installOglive", methods={"POST"})
     * @OA\Post(
     *     path="/ogboot/oglives/{isoName}",
     *     summary="Install an ogLive client",
     *     @OA\Parameter(
     *         name="isoName",
     *         in="path",
     *         description="Name of the ogLive ISO to install",
     *         required=true,
     *         @OA\Schema(
     *             type="string",
     *             example="ogLive-focal-5.13.0-27-beta-amd64-r20210706.5b4bf5f.iso"
     *         )
     *     ),
     *     @OA\Response(
     *         response=200,
     *         description="ogLive client installed successfully",
     *         @OA\JsonContent(
     *             type="object",
     *             @OA\Property(property="message", type="string")
     *         )
     *     ),
     *     @OA\Response(
     *         response=500,
     *         description="Failed to install ogLive client",
     *         @OA\JsonContent(
     *             type="object",
     *             @OA\Property(property="error", type="string")
     *         )
     *     )
     * )
     */
    public function installOglive(string $isoName): Response
    {
        # $result = $this->curlRequestService->common_request(OG_REST_CMD_POWEROFF, POST, [OG_REST_PARAM_CLIENTS => $ips]);
        $result = $this->curlRequestService->callOgLive("download $isoName");

        if ($result) {
            return new Response($result, Response::HTTP_OK);
        } else {
            return new Response('Failed', Response::HTTP_INTERNAL_SERVER_ERROR);
        }
    }

/**
 * @Route("/ogboot/oglives/{id}", name="uninstallOglive", methods={"DELETE"})
 * @OA\Delete(
 *     path="/ogboot/oglives/{id}",
 *     summary="Uninstall an ogLive client",
 *     @OA\Parameter(
 *         name="id",
 *         in="path",
 *         description="ID of the ogLive client to uninstall",
 *         required=true,
 *         @OA\Schema(
 *             type="integer"
 *         )
 *     ),
 *     @OA\Response(
 *         response=200,
 *         description="ogLive client uninstalled successfully",
 *         @OA\JsonContent(
 *             type="object",
 *             @OA\Property(property="message", type="string")
 *         )
 *     ),
 *     @OA\Response(
 *         response=500,
 *         description="Failed to uninstall ogLive client",
 *         @OA\JsonContent(
 *             type="object",
 *             @OA\Property(property="error", type="string")
 *         )
 *     )
 * )
 */

    public function uninstallOglive(Request $request): Response
    {
        $id = (int) $request->get('id');
        # $result = $this->curlRequestService->common_request(OG_REST_CMD_POWEROFF, POST, [OG_REST_PARAM_CLIENTS => $ips]);
        $result = $this->curlRequestService->callOgLive("uninstall $id");

        if ($result) {
            return new Response($result, Response::HTTP_OK);
        } else {
            return new Response('Failed', Response::HTTP_INTERNAL_SERVER_ERROR);
        }
    }

/**
 * @Route("/ogboot/v1/pxes", name="get_boot_files", methods={"GET"})
 * @OA\Get(
 *     path="/ogboot/v1/pxes",
 *     summary="Get all PXE boot files",
 *     @OA\Response(
 *         response=200,
 *         description="Successful operation",
 *         @OA\JsonContent(
 *             type="object",
 *             @OA\Property(
 *                 property="boot_files",
 *                 type="array",
 *                 @OA\Items(
 *                     type="string",
 *                     description="Boot file"
 *                 )
 *             )
 *         )
 *     ),
 *     @OA\Response(
 *         response=500,
 *         description="Failed to retrieve boot files"
 *     )
 * )
 */
public function getBootFiles(): JsonResponse
{

    $directory = '/opt/ogboot/tftpboot/ipxe_scripts';


    $files = scandir($directory);


    $bootFiles = array_filter($files, function ($file) use ($directory) {
        return !is_dir($directory . '/' . $file) && strpos($file, '01-') === 0;
    });

    $response = [
        'boot_files' => array_values($bootFiles)
    ];

    return new JsonResponse($response, Response::HTTP_OK);
}

    /**
     * @Route("/ogboot/v1/pxes/{mac}", name="ogboot_boot_file", methods={"GET"})
    * @OA\Get(
        *     path="/ogboot/v1/pxes/{mac}",
        *     summary="Obtener archivo de arranque PXE",
        *     description="Obtiene el contenido del archivo de arranque PXE para una dirección MAC específica.",
        *     @OA\Parameter(
        *         name="mac",
        *         in="path",
        *         required=true,
        *         description="Dirección MAC del cliente",
        *         @OA\Schema(type="string", example="00:50:56:22:11:12")
        *     ),
        *     @OA\Response(
        *         response=200,
        *         description="Contenido del archivo de arranque PXE",
        *         @OA\MediaType(
        *             mediaType="text/plain",
        *             @OA\Schema(type="string", example="Contenido del archivo PXE...")
        *         )
        *     ),
        *     @OA\Response(
        *         response=404,
        *         description="Archivo no encontrado"
        *     )
        * )
        */
    public function getBootFile(string $mac): Response
    {

        $directory = '/opt/ogboot/tftpboot/ipxe_scripts';


        $fileName = "01-" . $mac;


        $filePath = "$directory/$fileName";


        if (!file_exists($filePath)) {

            return new Response(null, Response::HTTP_NOT_FOUND);
        }

        $content = file_get_contents($filePath);

        return new Response($content, Response::HTTP_OK, ['Content-Type' => 'text/plain']);
    }


    /**
     * @Route("/ogboot/v1/pxes", name="create_boot_file", methods={"POST"})
     * @OA\Post(
     *     path="/ogboot/v1/pxes",
     *     summary="Create a new PXE boot file",
     *     @OA\RequestBody(
     *         required=true,
     *         @OA\JsonContent(
     *             type="object",
     *             @OA\Property(property="template_name", type="string", description="Template name", example="pxe"),
     *             @OA\Property(property="mac", type="string", description="MAC address", example="00:50:56:22:11:12"),
     *             @OA\Property(property="lang", type="string", description="Language", example="es_ES.UTF-8"),
     *             @OA\Property(property="ip", type="string", description="IP address", example="192.168.2.11"),
     *             @OA\Property(property="server_ip", type="string", description="Server IP address", example="192.168.2.1"),
     *             @OA\Property(property="router", type="string", description="Router", example="192.168.2.1"),
     *             @OA\Property(property="netmask", type="string", description="Netmask", example="255.255.255.0"),
     *             @OA\Property(property="computer_name", type="string", description="Computer name", example="pc11"),
     *             @OA\Property(property="netiface", type="string", description="Network interface", example="eth0"),
     *             @OA\Property(property="group", type="string", description="Group", example="Aula_virtual"),
     *             @OA\Property(property="ogrepo", type="string", description="Repository IP", example="192.168.2.1"),
     *             @OA\Property(property="oglive", type="string", description="Live server IP", example="192.168.2.1"),
     *             @OA\Property(property="oglog", type="string", description="Log server IP", example="192.168.2.1"),
     *             @OA\Property(property="ogshare", type="string", description="Share server IP", example="192.168.2.1"),
     *             @OA\Property(property="oglivedir", type="string", description="Live directory", example="ogLive"),
     *             @OA\Property(property="ogprof", type="string", description="Is professor", example="false"),
     *             @OA\Property(property="hardprofile", type="string", description="Hardware profile", example=""),
     *             @OA\Property(property="ogntp", type="string", description="NTP server", example=""),
     *             @OA\Property(property="ogdns", type="string", description="DNS server", example=""),
     *             @OA\Property(property="ogproxy", type="string", description="Proxy server", example=""),
     *             @OA\Property(property="ogunit", type="string", description="Unit directory", example=""),
     *             @OA\Property(property="resolution", type="string", description="Screen resolution", example="788")
     *         )
     *     ),
     *     @OA\Response(
     *         response=200,
     *         description="PXE boot file created successfully",
     *         @OA\JsonContent(
     *             type="object",
     *             @OA\Property(property="message", type="string")
     *         )
     *     ),
     *     @OA\Response(
     *         response=400,
     *         description="Invalid input"
     *     ),
     *     @OA\Response(
     *         response=404,
     *         description="Template not found"
     *     ),
     *     @OA\Response(
     *         response=500,
     *         description="Internal server error"
     *     )
     * )
     */
    public function createBootFile(Request $request): JsonResponse
    {
        $data = json_decode($request->getContent(), true);

        $templateName = $data['template_name'] ?? null;
        $mac = $data['mac'] ?? null;
        $serverIp = $data['server_ip'] ?? null;

        if (!$templateName || !$mac || !$serverIp) {
            return new JsonResponse(['error' => 'Invalid input'], Response::HTTP_BAD_REQUEST);
        }

        $parameters = [
            'LANG' => $data['lang'] ?? 'es_ES.UTF-8',
            'ip' => $data['ip'] ?? '',
            'router' => $data['router'] ?? '',
            'netmask' => $data['netmask'] ?? '',
            'computer_name' => $data['computer_name'] ?? '',
            'netiface' => $data['netiface'] ?? '',
            'group' => $data['group'] ?? '',
            'ogrepo' => $data['ogrepo'] ?? '',
            'oglive' => $data['oglive'] ?? $serverIp,
            'oglog' => $data['oglog'] ?? $serverIp,
            'ogshare' => $data['ogshare'] ?? $serverIp,
            'oglivedir' => $data['oglivedir'] ?? '',
            'ogprof' => $data['ogprof'] ?? 'false',
            'hardprofile' => $data['hardprofile'] ?? '',
            'ogntp' => $data['ogntp'] ?? '',
            'ogdns' => $data['ogdns'] ?? '',
            'ogproxy' => $data['ogproxy'] ?? '',
            'ogunit' => $data['ogunit'] ?? '',
            'resolution' => $data['resolution'] ?? '788'
        ];

        $templateDir = '/opt/ogboot/tftpboot/ipxe_scripts/templates';
        $templatePath = $templateDir . '/' . $templateName;

        if (!file_exists($templatePath)) {
            return new JsonResponse(['error' => 'Template not found'], Response::HTTP_NOT_FOUND);
        }

        $templateContent = file_get_contents($templatePath);
        if ($templateContent === false) {
            return new JsonResponse(['error' => 'Failed to read template'], Response::HTTP_INTERNAL_SERVER_ERROR);
        }

        $kernelArgs = 'ro boot=oginit quiet splash irqpoll acpi=on og2nd=sqfs ogprotocol=smb ogactiveadmin=true ogdebug=true ogtmpfs=15 ' .
            'oglivedir=' . $parameters['oglivedir'] . ' ' .
            'LANG=' . $parameters['LANG'] . ' ' .
            'ip=' . $parameters['ip'] . ':' . $serverIp . ':' . $parameters['router'] . ':' . $parameters['netmask'] . ':' . $parameters['computer_name'] . ':' . $parameters['netiface'] . ':none ' .
            'group=' . str_replace(' ', '_', trim($parameters['group'])) . ' ' .
            'ogrepo=' . $parameters['ogrepo'] . ' ' .
            'oglive=' . $serverIp . ' ' .
            'oglog=' . $serverIp . ' ' .
            'ogshare=' . $serverIp . ' ' .
            'ogprof=' . ($parameters['ogprof'] === 'true' ? 'true' : 'false') . ' ' .
            (!empty($parameters['hardprofile']) ? 'hardprofile=' . str_replace(' ', '_', trim($parameters['hardprofile'])) . ' ' : '') .
            (!empty($parameters['ogntp']) ? 'ogntp=' . $parameters['ogntp'] . ' ' : '') .
            (!empty($parameters['ogdns']) ? 'ogdns=' . $parameters['ogdns'] . ' ' : '') .
            (!empty($parameters['ogproxy']) ? 'ogproxy=' . $parameters['ogproxy'] . ' ' : '') .
            (!empty($parameters['ogunit']) ? 'ogunit=' . $parameters['ogunit'] . ' ' : '') .
            (is_numeric($parameters['resolution']) && $parameters['resolution'] <= 999 ? 'vga=' . $parameters['resolution'] :
                (strpos($parameters['resolution'], ':') !== false ? 'video=' . $parameters['resolution'] : ' ' . $parameters['resolution']));


        $pxeContent = str_replace(['__INFOHOST__', '__SERVERIP__'], [$kernelArgs, $serverIp], $templateContent);


        $pxeFileName = '01-' . $mac; 

        $pxeFilePath = '/opt/ogboot/tftpboot/ipxe_scripts/' . $pxeFileName;

        if (file_put_contents($pxeFilePath, $pxeContent) === false) {
            return new JsonResponse(['error' => 'Failed to create PXE boot file'], Response::HTTP_INTERNAL_SERVER_ERROR);
        }

        return new JsonResponse(['message' => 'PXE boot file created successfully'], Response::HTTP_OK);
    }
    /**
     * @Route("/ogboot/v1/pxes/{mac}", name="ogboot_delete_boot_file", methods={"DELETE"})
     */
    public function deleteBootFile(string $mac): Response
    {
        $directory = '/opt/ogboot/tftpboot/ipxe_scripts';

        $fileName = "01-" . str_replace(':', '-', $mac);

        $filePath = "$directory/$fileName";

        if (!file_exists($filePath)) {
            
            return new JsonResponse(['error' => 'No se encontró ningún archivo de arranque con la dirección MAC especificada'], Response::HTTP_NOT_FOUND);
        }

        if (!unlink($filePath)) {
            
            return new JsonResponse(['error' => 'Error al eliminar el archivo de arranque'], Response::HTTP_INTERNAL_SERVER_ERROR);
        }
        return new JsonResponse(['message' => 'El archivo de arranque se eliminó correctamente'], Response::HTTP_OK);
    }
    

    /**
     * @Route("/ogboot/v1/pxe-templates", methods={"GET"})
     * @OA\Get(
     *     path="/ogboot/v1/pxe-templates",
     *     summary="Obtener todas las plantillas PXE",
     *     description="Obtiene una lista de todas las plantillas de arranque PXE disponibles.",
     *     @OA\Response(
     *         response=200,
     *         description="Lista de plantillas de arranque PXE",
     *         @OA\JsonContent(
     *             type="object",
     *             @OA\Property(
     *                 property="templates",
     *                 type="array",
     *                 @OA\Items(type="string", example="mi_plantilla.ipxe")
     *             )
     *         )
     *     ),
     *     @OA\Response(
     *         response=500,
     *         description="Error interno del servidor"
     *     )
     * )
     */
    public function getAllTemplates(): JsonResponse
    {
        $templateDir = '/opt/ogboot/tftpboot/ipxe_scripts/templates';

        // Verificar si el directorio existe
        if (!is_dir($templateDir)) {
            return new JsonResponse(['error' => 'Directorio de plantillas no encontrado'], Response::HTTP_INTERNAL_SERVER_ERROR);
        }

        // Obtener la lista de archivos en el directorio
        $files = scandir($templateDir);

        // Filtrar los archivos válidos (excluir directorios y archivos ocultos)
        $templates = array_filter($files, function ($file) use ($templateDir) {
            return !in_array($file, ['.', '..']) && !is_dir($templateDir . '/' . $file);
        });

        return new JsonResponse(['templates' => array_values($templates)], Response::HTTP_OK);
    }

    /**
     * @Route("/ogboot/v1/pxe-templates/{templateName}", name="get_template", methods={"GET"})
     * @OA\Get(
     *     path="/ogboot/v1/pxe-templates/{templateName}",
     *     summary="Obtener contenido de una plantilla PXE",
     *     description="Obtiene el contenido de una plantilla de arranque PXE específica por su nombre.",
     *     @OA\Parameter(
     *         name="templateName",
     *         in="path",
     *         required=true,
     *         description="Nombre de la plantilla PXE",
     *         @OA\Schema(type="string", example="mi_plantilla.ipxe")
     *     ),
     *     @OA\Response(
     *         response=200,
     *         description="Contenido de la plantilla de arranque PXE",
     *         @OA\MediaType(
     *             mediaType="text/plain",
     *             @OA\Schema(type="string", example="#!ipxe\nset timeout 0\nset timeout-style hidden\n\nset ISODIR ogLive\nset default 0\nset kernelargs INFOHOST\nkernel tftp://SERVERIP/ogLive/ogvmlinuz ${kernelargs}\ninitrd tftp://SERVERIP/ogLive/oginitrd.img\nboot")
     *         )
     *     ),
     *     @OA\Response(
     *         response=404,
     *         description="Plantilla no encontrada"
     *     ),
     *     @OA\Response(
     *         response=500,
     *         description="Error interno del servidor"
     *     )
     * )
     */
    public function getTemplate(string $templateName): Response
    {
        $templateDir = '/opt/ogboot/tftpboot/ipxe_scripts/templates';
        $filePath = "$templateDir/$templateName";

        if (!file_exists($filePath)) {
            return new Response(null, Response::HTTP_NOT_FOUND);
        }

        $content = file_get_contents($filePath);
        if ($content === false) {
            return new Response('Error al leer la plantilla', Response::HTTP_INTERNAL_SERVER_ERROR);
        }

        return new Response($content, Response::HTTP_OK, ['Content-Type' => 'text/plain']);
    }

    /**
     * @Route("/ogboot/v1/pxe-templates", methods={"POST"})
     * 
     * @OA\Post(
     *     path="/ogboot/v1/pxe-templates",
     *     summary="Crear Plantilla",
     *     description="Crea una nueva plantilla de arranque utilizando los datos proporcionados.",
     *     @OA\RequestBody(
     *         required=true,
     *         @OA\JsonContent(
     *             type="object",
     *             @OA\Property(property="name_template", type="string", example="pxe"),
     *             @OA\Property(property="content_template", type="string", example="#!ipxe\nset timeout 0\nset timeout-style hidden\n\nset ISODIR ogLive\nset default 0\nset kernelargs INFOHOST\nkernel tftp://SERVERIP/ogLive/ogvmlinuz ${kernelargs}\ninitrd tftp://SERVERIP/ogLive/oginitrd.img\nboot")
     *         )
     *     ),
     *     @OA\Response(
     *         response=200,
     *         description="La plantilla de arranque se creó exitosamente.",
     *         @OA\JsonContent(
     *             type="object",
     *             @OA\Property(property="message", type="string", example="Plantilla creada exitosamente."),
     *             @OA\Property(property="template", type="string", example="#!ipxe\nset timeout 0\nset timeout-style hidden\n\nset ISODIR ogLive\nset default 0\nset kernelargs INFOHOST\nkernel tftp://SERVERIP/ogLive/ogvmlinuz ${kernelargs}\ninitrd tftp://SERVERIP/ogLive/oginitrd.img\nboot")
     *         )
     *     ),
     *     @OA\Response(
     *         response=400,
     *         description="La solicitud no pudo ser procesada debido a un error en los datos proporcionados en el cuerpo de la solicitud."
     *     ),
     *     @OA\Response(
     *         response=500,
     *         description="Ocurrió un error al crear la plantilla de arranque."
     *     )
     * )
     */
    public function createTemplate(Request $request)
    {
        $data = json_decode($request->getContent(), true);

        if (!isset($data['name_template']) || !isset($data['content_template'])) {
            return new Response('La solicitud no pudo ser procesada debido a un error en los datos proporcionados en el cuerpo de la solicitud.', 400);
        }

        $nameTemplate = $data['name_template'];
        $contentTemplate = $data['content_template'];
        $templateDir = '/opt/ogboot/tftpboot/ipxe_scripts/templates';
        if (!preg_match('/^[a-zA-Z0-9._-]+$/', $nameTemplate)) {
            return new Response('Nombre de la plantilla no válido.', 400);
        }

        $filePath = $templateDir . '/' . $nameTemplate;
        $contentTemplate = str_replace("\\n", "\n", $contentTemplate);
        try {
            file_put_contents($filePath, $contentTemplate);
        } catch (\Exception $e) {
            return new Response('Ocurrió un error al crear la plantilla de arranque.', 500);
        }

        return new Response(json_encode([
            'message' => 'Plantilla creada exitosamente.',
            'template' => $contentTemplate
        ]), 200, ['Content-Type' => 'application/json']);
    }



   /**
     * @Route("/ogboot/v1/pxe-templates/{name}", methods={"DELETE"})
     * 
     * @OA\Delete(
     *     path="/ogboot/v1/pxe-templates/{name}",
     *     summary="Eliminar Plantilla",
     *     description="Elimina una plantilla de arranque específica utilizando su nombre.",
     *     @OA\Parameter(
     *         name="name",
     *         in="path",
     *         required=true,
     *         description="El nombre de la plantilla de arranque que se desea eliminar.",
     *         @OA\Schema(type="string")
     *     ),
     *     @OA\Response(
     *         response=200,
     *         description="La plantilla de arranque se eliminó correctamente.",
     *         @OA\JsonContent(
     *             type="object",
     *             @OA\Property(property="message", type="string", example="Plantilla eliminada correctamente.")
     *         )
     *     ),
     *     @OA\Response(
     *         response=404,
     *         description="No se encontró ninguna plantilla de arranque con el nombre especificado."
     *     ),
     *     @OA\Response(
     *         response=500,
     *         description="Ocurrió un error al eliminar la plantilla de arranque."
     *     )
     * )
     */
    public function deleteTemplate($name)
    {
        $templateDir = '/opt/ogboot/tftpboot/ipxe_scripts/templates';
        $filePath = $templateDir . '/' . $name;

        if (!file_exists($filePath)) {
            return new Response('No se encontró ninguna plantilla de arranque con el nombre especificado.', 404);
        }

        try {
            unlink($filePath);
        } catch (\Exception $e) {
            return new Response('Ocurrió un error al eliminar la plantilla de arranque.', 500);
        }

        return new Response(json_encode([
            'message' => 'Plantilla eliminada correctamente.'
        ]), 200, ['Content-Type' => 'application/json']);
    }


}
